from models.billet_counter import Counter
from models.data_sender import Sender
from models.billet_trace_pusher import Trace_pusher
from utils.s7data import S7data, S7Client
from utils.mqttdata import Mqttdata, MqttClient
from utils.logger import Logger


##############################################################
# 日志配置

logger_5 = Logger('5#')
logger_5.file_on('logs/5#log.log')
logger_5.screen_on()

logger_6 = Logger('6#')
logger_6.file_on('logs/6#log.log')
logger_6.screen_on()

logger_sender = Logger('sender')
logger_sender.file_on('logs/sender_log.log')
logger_sender.screen_on()


##############################################################
# S7数据源配置

s7_5 = S7Client()
s7_5.connect('192.168.1.215', 0, 0)
data_5 = S7data(r"conf/5#nodes.csv")
data_5.set_S7Client(s7_5)
data_5.set_logger(logger_5)
data_5.start_auto_update()
logger_5.info('[PREPARE]5#机PLC连接成功')

s7_6 = S7Client()
s7_6.connect('192.168.1.216', 0, 0)
data_6 = S7data(r"conf/6#nodes.csv")
data_6.set_S7Client(s7_6)
data_6.set_logger(logger_6)
data_6.start_auto_update()
logger_6.info('[PREPARE]6#机PLC连接成功')


##############################################################
# MQTT数据源配置

mqtt_mes = MqttClient('python-mqtt-biller_digitalization_test3', 'readonly', '1qazxsw@')
mqtt_mes.connect('192.168.12.201', 1883)
mqtt_mes.loop_start()
data_mes = Mqttdata()
data_mes.set_mqtt_client(mqtt_mes)
logger_5.info('[PREPARE]与MES使用MQTT连接成功')
logger_6.info('[PREPARE]与MES使用MQTT连接成功')

mqtt_web = MqttClient('python-mqtt-992_test3')
mqtt_web.connect('192.168.0.119', 1883)
mqtt_web.loop_start()
data_web = Mqttdata()
data_web.set_mqtt_client(mqtt_web)
logger_sender.info('[PREPARE]与WEB业务平台使用MQTT连接成功')


##############################################################
# 数据发送服务

sender = Sender(logger_sender)
#sender.set_mqtt_client(mqtt_web)

# debug设置
sender.http_flag = False


##############################################################
# 分炉分坯服务

position_5 = [10462, 11553, 12671, 14020, 31641, 16412, 17229, 18809]
position_6 = [7610, 9175, 10425, 11700, 13000, 14350, 15577, 16952]

pusher_5 = Trace_pusher(data_5, logger_5, sender, position_5, True)
pusher_6 = Trace_pusher(data_6, logger_6, sender, position_6)

flfp_5 = Counter(data_mes, data_5, 5, logger_5, sender, pusher_5)
flfp_6 = Counter(data_mes, data_6, 6, logger_6, sender, pusher_6)
